/*
  Datatabase Update Script
  From 3.0.x series to 3.1.x series
*/

-- New tables (Bug types and priorities
CREATE TABLE [dbo].[TT_Priorities] (
	[BugPriorityID] [int] IDENTITY (1, 1) NOT NULL ,
	[ProjectID] [int] NOT NULL ,
	[Number] [int] NOT NULL ,
	[Name] [nvarchar] (100) COLLATE Cyrillic_General_CI_AS NOT NULL 
) ON [PRIMARY]

GO

ALTER TABLE [dbo].[TT_Priorities] WITH NOCHECK ADD 
	CONSTRAINT [PK_TT_Priorities] PRIMARY KEY  CLUSTERED 
	(
		[BugPriorityID]
	)  ON [PRIMARY] 
GO

ALTER TABLE [dbo].[TT_Priorities] ADD 
	CONSTRAINT [FK_TT_Priorities_TT_Projects] FOREIGN KEY 
	(
		[ProjectID]
	) REFERENCES [dbo].[TT_Projects] (
		[ProjectID]
	) ON DELETE CASCADE 
GO

CREATE TABLE [dbo].[TT_BugTypes] (
	[BugTypeID] [int] IDENTITY (1, 1) NOT NULL ,
	[ProjectID] [int] NOT NULL ,
	[Number] [int] NOT NULL ,
	[Name] [nvarchar] (100) COLLATE Cyrillic_General_CI_AS NOT NULL 
) ON [PRIMARY]

GO

ALTER TABLE [dbo].[TT_BugTypes] WITH NOCHECK ADD 
	CONSTRAINT [PK_TT_BugTypes] PRIMARY KEY  CLUSTERED 
	(
		[BugTypeID]
	)  ON [PRIMARY] 
GO

ALTER TABLE [dbo].[TT_BugTypes] ADD 
	CONSTRAINT [FK_TT_BugTypes_TT_Projects] FOREIGN KEY 
	(
		[ProjectID]
	) REFERENCES [dbo].[TT_Projects] (
		[ProjectID]
	) ON DELETE CASCADE 
GO

-- for each project add default objects
DECLARE @projid int

DECLARE projcur CURSOR FOR 
	SELECT ProjectID
	FROM TT_Projects
	ORDER BY ProjectID ASC

OPEN projcur

FETCH NEXT FROM projcur
INTO @projid

WHILE @@FETCH_STATUS = 0
BEGIN
	/*
	  default priorities 
	  1 - Urgent, 2 - high, 3 - medium, 4 - low
	*/

	INSERT INTO TT_Priorities VALUES(@projid, 1, 'Urgent')
	INSERT INTO TT_Priorities VALUES(@projid, 2, 'High')
	INSERT INTO TT_Priorities VALUES(@projid, 3, 'Medium')
	INSERT INTO TT_Priorities VALUES(@projid, 4, 'Low')

	/*
	  default bug types
	  1-interface, 2-misspelling, 3-design, 
	  4-functionality, 5-logic, 6-calculation
	*/
	INSERT INTO TT_BugTypes VALUES(@projid, 1, 'Interface')
	INSERT INTO TT_BugTypes VALUES(@projid, 2, 'Misspelling')
	INSERT INTO TT_BugTypes VALUES(@projid, 3, 'Design')
	INSERT INTO TT_BugTypes VALUES(@projid, 4, 'Functionality')
	INSERT INTO TT_BugTypes VALUES(@projid, 5, 'Logic')
	INSERT INTO TT_BugTypes VALUES(@projid, 6, 'Calculation')

	FETCH NEXT FROM projcur
	INTO @projid
END

CLOSE projcur
DEALLOCATE projcur
GO

ALTER TABLE TT_Projects ADD Priority int NULL
GO
UPDATE TT_Projects SET Priority = 1

ALTER TABLE TT_UserOps ADD ShowCompletedTasks bit NULL
GO

UPDATE TT_UserOps SET ShowCompletedTasks=1

-- surrogate PK in the TT_ProjectMembers table
CREATE TABLE [dbo].[TT_ProjectMembers_t] (
	[ProjectID] [int] NOT NULL ,
	[UserID] [int] NOT NULL ,
	[RoleID] [int] NOT NULL ,
	[Security] [nvarchar] (4000) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
) ON [PRIMARY]
GO

INSERT INTO TT_ProjectMembers_t
SELECT ProjectID, UserID, RoleID, Security
FROM TT_ProjectMembers
GO

DELETE FROM TT_ProjectMembers
GO

ALTER TABLE [dbo].[TT_ProjectMembers] DROP CONSTRAINT PK_ProjectMembers
GO

ALTER TABLE [dbo].[TT_ProjectMembers] ADD [MemberID] [int] IDENTITY (1, 1) NOT NULL 
GO

INSERT INTO TT_ProjectMembers
SELECT ProjectID, UserID, RoleID, Security
FROM TT_ProjectMembers_t
GO

ALTER TABLE [dbo].[TT_ProjectMembers] WITH NOCHECK ADD 
	CONSTRAINT [PK_ProjectMembers] PRIMARY KEY  CLUSTERED 
	(
		[MemberID]
	)  ON [PRIMARY] 
GO

DROP TABLE TT_ProjectMembers_t
GO
	