-- normalizing workflow:
-- 1. ensuring there's only one record related to object creation
-- 2. removing deleted objects records

	
delete from TT_Workflow where 
(  
	NOT EXISTS(select ProjectID from TT_Projects where TT_Projects.ProjectID = TT_Workflow.ObjectID) 
		AND TT_Workflow.ObjectType=1
)

delete from TT_Workflow where 
(
	NOT EXISTS(select BugID from TT_Bugs where TT_Bugs.BugID = TT_Workflow.ObjectID) 
		AND TT_Workflow.ObjectType=2
) 

delete from TT_Workflow where 
(
	NOT EXISTS(select TaskID from TT_Tasks where TT_Tasks.TaskID = TT_Workflow.ObjectID) 
		AND TT_Workflow.ObjectType=3 
) 

delete from TT_Workflow where 
(
	NOT EXISTS(select ReqID from TT_Requests where TT_Requests.ReqID = TT_Workflow.ObjectID) 
		AND TT_Workflow.ObjectType=4 
) 

delete from TT_Workflow where 
(
	NOT EXISTS(select AttachID from TT_Attach where TT_Attach.AttachID = TT_Workflow.ObjectID) 
		AND TT_Workflow.ObjectType=5 
) 

delete from TT_Workflow where 
(
	NOT EXISTS(select MessageID from TT_MessageBoard where TT_MessageBoard.MessageID = TT_Workflow.ObjectID) 
		AND TT_Workflow.ObjectType=6 
)

declare @id int
declare @UserID int
declare @cnt int
declare @WflID int
declare @dt datetime

/*
	the following block makes sure that only one "create" workflow entry
	is there for bugs
*/
declare bugCur cursor for
	select BugID, OpenedBy, DateOpened from TT_Bugs

open bugCur

fetch next from bugCur into @id, @UserID, @dt

while @@fetch_Status = 0
begin
	set @cnt = 
	(
		select count(WflID) from TT_Workflow
		where Action=1 and CustomAction=0 and ObjectType=2 and ObjectID=@id
	)

	if @cnt = 0
		insert into TT_Workflow(Action, CustomAction, ObjectID, ObjectType, UserID, Description, EntryDate, OwnerID)
			values(1, 0, @id, 2, @UserID, 'This bug was added', @dt, @UserID)

	if @cnt > 1 
	begin
		set @WflID = 
		(
			select top 1 WflID from TT_Workflow
			where Action=1 and CustomAction=0 and ObjectType=2 and ObjectID=@id
		)

		delete from TT_Workflow 
		where Action=1 and CustomAction=0 and ObjectType=2 and ObjectID=@id and WflID<>@WflID
	end
	
	fetch next from bugCur into @id, @UserID, @dt
end

close bugCur 
deallocate bugCur

/*
	the following block makes sure that only one "create" workflow entry
	is there for requests
*/
declare reqCur cursor for
	select ReqID, DateEntered from TT_Requests

open reqCur

fetch next from reqCur into @id, @dt

while @@fetch_Status = 0
begin
	set @cnt = 
	(
		select count(WflID) from TT_Workflow
		where Action=1 and CustomAction=0 and ObjectType=4 and ObjectID=@id
	)

	if @cnt = 0
		insert into TT_Workflow(Action, CustomAction, ObjectID, ObjectType, UserID, Description, EntryDate, OwnerID)
			values(1, 0, @id, 4, 1, 'This request was added', @dt, 1)

	if @cnt > 1 
	begin
		set @WflID = 
		(
			select top 1 WflID from TT_Workflow
			where Action=1 and CustomAction=0 and ObjectType=4 and ObjectID=@id
		)

		delete from TT_Workflow 
		where Action=1 and CustomAction=0 and ObjectType=4 and ObjectID=@id and WflID<>@WflID
	end
	
	fetch next from reqCur into @id, @dt
end

close reqCur 
deallocate reqCur

/*
	normalize task workflow
*/

declare taskCur cursor for
	select TaskID from TT_Tasks

open taskCur

fetch next from taskCur into @id

while @@fetch_Status = 0
begin
	set @cnt = 
	(
		select count(WflID) from TT_Workflow
		where Action=1 and CustomAction=0 and ObjectType=3 and ObjectID=@id
	)

	if @cnt = 0
		insert into TT_Workflow(Action, CustomAction, ObjectID, ObjectType, UserID, Description, EntryDate, OwnerID)
			values(1, 0, @id, 3, 1, 'This task was added', GETDATE(), 1)

	if @cnt > 1 
	begin
		set @WflID = 
		(
			select top 1 WflID from TT_Workflow
			where Action=1 and CustomAction=0 and ObjectType=3 and ObjectID=@id
		)

		delete from TT_Workflow 
		where Action=1 and CustomAction=0 and ObjectType=3 and ObjectID=@id and WflID<>@WflID
	end
	
	fetch next from taskCur into @id
end

close taskCur 
deallocate taskCur


alter table TT_Tasks
	add 
	TaskType int NULL,
	BugType int NULL


GO

update TT_Tasks set TaskType=1
update TT_Tasks set TaskType=2, TT_Tasks.Status = (select TT_Bugs.Status from TT_Bugs where TaskID=TT_Tasks.TaskID) where exists(select BugID from TT_Bugs where TaskID=TT_Tasks.TaskID)
update TT_Tasks set TaskType=3, TT_Tasks.Status = (select TT_Requests.Status from TT_Requests where TaskID=TT_Tasks.TaskID) where exists(select ReqID from TT_Requests where TaskID=TT_Tasks.TaskID)

alter table TT_Tasks
	alter column TaskType int NOT NULL

/*
	II. remove assigned bugs - they already have appropriate tasks
  delete from TT_Bugs where TaskID<>0
 
	III. remove assigned requests - they already have appropriate tasks
  delete from TT_Requests where TaskID<>0
*/
 
/*
	IV. copy unassigned bugs to tasks
*/

declare bugCreateCur cursor for
	select BugID, ProjectID, Description, OpenedBy, ClosedBy, Priority,
		DateOpened, BugType, Status, Summary
	from TT_Bugs WHERE TaskID=0

declare @id int,
	@projid int,
	@descr nvarchar(4000),
	@openedby int,
	@closedby int,
	@priority int,
	@dateopened datetime,
	@bugtype int,
	@Status int,
	@summary nvarchar(100),
	@bugCatId int,
	@newstat int

declare @statnum int,
	@statname nvarchar(50),
	@statcomplete bit

open bugCreateCur

fetch next from bugCreateCur into
	@id, @projid, @descr, @openedby, @closedby, @priority,
	@dateopened, @bugtype, @Status, @summary

while @@fetch_Status = 0
begin
	-- read default category for bugs
	set @bugCatId = (select BugCategoryID from TT_Projects where ProjectID=@projid)

	-- read bug Status Description. if Status is not present in task Statuses,
	-- add it to the Status table
	select @statnum = Number, @statname = StatusName, @statcomplete = TreatAsCompleted
	from DD_Statuses where StatusType = 'Bug' and Number = @Status

	if not exists(select ID from DD_Statuses where StatusType = 'Bug' and StatusName = @statname)
	begin
		-- get new available Status number
		set @newstat = (select max(Number) from DD_Statuses where StatusType = 'Bug') + 1
	
		-- insert bug Status as a new
		insert into DD_Statuses(StatusName, Number, StatusType, TreatAsCompleted)
		values(@statname, @newstat, 'Bug', @statcomplete)
	end
	else
		set @newstat = (select Number from DD_Statuses where StatusType='Bug' and StatusName = @statname)

	-- insert a new task record
	insert into TT_Tasks(ProjectID, Description, ParentID, Status, IsOutOfScope, CategoryID, Title, ChangeStatus, Billable,
		BillType, Amount, PercentCompleted, Priority, TaskType, BugType)
	values(@projid, @descr, 0, @newstat, 0, @bugCatId, @summary, 0, 
		1, 1, 0, 0, @priority, 2, @bugtype)

	-- update messageboard
	update TT_MessageBoard
	set ParentID=@@IDENTITY, ParentType=1
	where (ParentID=@id) and (ParentType=2)

	-- update attachments
	update TT_Attach
	set ParentID=@@IDENTITY, ParentType=1
	where (ParentID=@id) and (ParentType=2)

	set @id = @@IDENTITY

	-- add a workflow entry for it
	insert into TT_Workflow(Action, CustomAction, ObjectID, ObjectType, UserID,
		Description, EntryDate, OwnerID)
	values(1, 0, @id, 3, @openedby, 'This task was added', @dateopened, @openedby)

	fetch next from bugCreateCur into
		@id, @projid, @descr, @openedby, @closedby, @priority,
		@dateopened, @bugtype, @Status, @summary
end

close bugCreateCur
deallocate bugCreateCur

GO

/*
	IV. copy unassigned requests to tasks
*/

declare reqCreateCur cursor for
	select rq.ReqID, rq.ProjectID, rq.Description, wfl.UserID createdby, 
		wfl.EntryDate creationdate, rq.Status, rq.Summary
	from TT_Requests rq
	inner join TT_Workflow wfl on rq.ReqID=wfl.ObjectID and wfl.ObjectType=4 and wfl.Action = 1 and wfl.CustomAction = 0
	WHERE TaskID=0

declare @id int,
	@projid int,
	@descr nvarchar(4000),
	@createdby int,
	@priority int,
	@creationdate datetime,
	@Status int,
	@summary nvarchar(100),
	@reqCatId int,
	@newstat int

declare @statnum int,
	@statname nvarchar(50),
	@statcomplete bit

open reqCreateCur

fetch next from reqCreateCur into
	@id, @projid, @descr, @createdby, 
	@creationdate, @Status, @summary

while @@fetch_Status = 0
begin
	-- read default category for requests
	set @reqCatId = (select ReqCategoryID from TT_Projects where ProjectID=@projid)

	-- read request Status Description. if Status is not present in task Statuses,
	-- add it to the Status table
	select @statnum = Number, @statname = StatusName, @statcomplete = TreatAsCompleted
	from DD_Statuses where StatusType = 'Request' and Number = @Status

	if not exists(select ID from DD_Statuses where StatusType = 'Request' and StatusName = @statname)
	begin
		-- get new available Status number
		set @newstat = (select max(Number) from DD_Statuses where StatusType = 'Request') + 1
	
		-- insert request Status as a new
		insert into dd_Statuses(StatusName, Number, StatusType, TreatAsCompleted)
		values(@statname, @newstat, 'Request', @statcomplete)
	end
	else
		set @newstat = (select Number from DD_Statuses where StatusType='Request' and StatusName = @statname)

	-- insert a new task record
	insert into TT_Tasks(ProjectID, Description, ParentID, Status, IsOutOfScope, CategoryID, Title, ChangeStatus, 
		Billable, BillType, Amount, PercentCompleted, Priority, TaskType)
	values(@projid, @descr, 0, @newstat, 0, @reqCatId, @summary, 0, 
		1, 1, 0, 0, 3, 3)

	-- update messageboard
	update TT_MessageBoard
	set ParentID=@@IDENTITY, ParentType=1
	where (ParentID=@id) and (ParentType=3)

	-- update attachments
	update TT_Attach
	set ParentID=@@IDENTITY, ParentType=1
	where (ParentID=@id) and (ParentType=3)

	set @id = @@IDENTITY

	-- add a workflow entry for it
	insert into TT_Workflow(Action, CustomAction, ObjectID, ObjectType, UserID,
		Description, EntryDate, OwnerID)
	values(1, 0, @id, 3, @createdby, 'This task was added', @creationdate, @createdby)

	fetch next from reqCreateCur into
		@id, @projid, @descr, @createdby, 
		@creationdate, @Status, @summary
end

GO

ALTER TABLE TT_CustomerDetails ADD Status bit NULL
GO

ALTER TABLE TT_CustomerDetails 
	DROP CONSTRAINT FK_TT_CustomerDetails_TT_Users
GO

ALTER TABLE TT_CustomerDetails 
	DROP COLUMN UserID
GO

ALTER TABLE TT_Users ADD CustomerID int NULL
GO

ALTER TABLE TT_Projects ADD CustomerID int NULL
GO

UPDATE TT_CustomerDetails SET Status=1

INSERT INTO TT_Roles(Name, Security, Description, HourlyRate) 
VALUES('Customer - Limited Access', '<security><right name="ViewAllProjects" value="off" /><right name="ViewAllTasks" value="off" /><right name="ViewAllBugs" value="off" /><right name="ViewAllRequests" value="off" /><right name="ViewAllTimeEntries" value="off" /><right name="ViewOwnProjects" value="on" /><right name="ViewOwnTasks" value="off" /><right name="ViewOwnBugs" value="off" /><right name="ViewOwnRequests" value="off" /><right name="ViewOwnTimeEntries" value="off" /><right name="EditAllProjects" value="off" /><right name="EditAllTasks" value="off" /><right name="EditAllBugs" value="off" /><right name="EditAllRequests" value="off" /><right name="EditOwnProjects" value="off" /><right name="EditOwnTasks" value="off" /><right name="EditOwnBugs" value="off" /><right name="EditOwnRequests" value="off" /><right name="DeleteAllProjects" value="off" /><right name="DeleteAllTasks" value="off" /><right name="DeleteAllBugs" value="off" /><right name="DeleteAllRequests" value="off" /><right name="DeleteOwnProjects" value="off" /><right name="DeleteOwnTasks" value="off" /><right name="DeleteOwnBugs" value="off" /><right name="DeleteOwnRequests" value="off" /><right name="AddProjects" value="off" /><right name="AddTasks" value="off" /><right name="AddBugs" value="off" /><right name="AddRequests" value="off" /><right name="AddTimeEntries" value="off" /><right name="ShowCustomerReport" value="off" /><right name="ShowSummaryReport" value="off" /><right name="ShowBugReport" value="off" /><right name="ShowRequestReport" value="off" /><right name="ShowResourceReport" value="off" /><right name="ShowProjectReport" value="off" /><right name="Assign" value="off" /><right name="ManageUsers" value="off" /></security>', 'built-in role', 12)

UPDATE TT_Roles SET Name = 'Customer - Full Access' WHERE RoleID = 4 


-- delete all tasks which have fake categories
delete from TT_Tasks where CategoryID not in (select CategoryID from TT_Categories)


BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.DD_TaskTypes Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

CREATE TABLE [dbo].[DD_TaskTypes] (
   [TypeName] [nvarchar] (25) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
   [ID] [int] NOT NULL
)
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.DD_TaskTypes Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.DD_TaskTypes Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.MD_Fields Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

CREATE TABLE [dbo].[MD_Fields] (
   [FieldID] [int] IDENTITY (1, 1) NOT NULL,
   [EntityID] [int] NOT NULL,
   [FieldName] [nvarchar] (50) COLLATE Cyrillic_General_CI_AS NOT NULL,
   [DisplayName] [nvarchar] (50) COLLATE Cyrillic_General_CI_AS NULL,
   [DataType] [int] NOT NULL
)
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[MD_Fields] ADD CONSTRAINT [PK_MD_Fields] PRIMARY KEY CLUSTERED ([FieldID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.MD_Fields Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.MD_Fields Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_Actions Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

CREATE TABLE [dbo].[NF_Actions] (
   [ActionID] [int] IDENTITY (1, 1) NOT NULL,
   [Operation] [char] (10) COLLATE Cyrillic_General_CI_AS NOT NULL,
   [Target] [varchar] (25) COLLATE Cyrillic_General_CI_AS NOT NULL
)
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[NF_Actions] ADD CONSTRAINT [PK_NF_Actions] PRIMARY KEY CLUSTERED ([ActionID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_Actions Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_Actions Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_Notifications Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

CREATE TABLE [dbo].[NF_Notifications] (
   [NotificationID] [int] IDENTITY (1, 1) NOT FOR REPLICATION NOT NULL,
   [ProjectID] [int] NOT NULL,
   [PM] [bit] NOT NULL,
   [AU] [bit] NOT NULL,
   [Title] [varchar] (100) COLLATE Cyrillic_General_CI_AS NOT NULL
)
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[NF_Notifications] ADD CONSTRAINT [PK_NF_Notifications] PRIMARY KEY CLUSTERED ([NotificationID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_Notifications Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_Notifications Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_NotificationDetails Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

CREATE TABLE [dbo].[NF_NotificationDetails] (
   [NotificationDetailID] [int] IDENTITY (1, 1) NOT NULL,
   [NotificationID] [int] NOT NULL,
   [ActionID] [int] NOT NULL,
   [Subject] [varchar] (200) COLLATE Cyrillic_General_CI_AS NULL,
   [Message] [varchar] (500) COLLATE Cyrillic_General_CI_AS NULL
)
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[NF_NotificationDetails] ADD CONSTRAINT [PK_NF_NotificationDetails] PRIMARY KEY CLUSTERED ([NotificationDetailID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_NotificationDetails Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_NotificationDetails Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_NotificationUsers Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

CREATE TABLE [dbo].[NF_NotificationUsers] (
   [NotificationUserID] [int] IDENTITY (1, 1) NOT NULL,
   [NotificationID] [int] NOT NULL,
   [UserID] [int] NOT NULL
)
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[NF_NotificationUsers] ADD CONSTRAINT [PK_NF_NotificationUsers] PRIMARY KEY CLUSTERED ([NotificationUserID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_NotificationUsers Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_NotificationUsers Table'
END
GO


BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Deleting dbo.TT_BugTypes Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   DROP TABLE [dbo].[TT_BugTypes]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_BugTypes Table Deleted Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Delete dbo.TT_BugTypes Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Updating dbo.TT_Users Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO


IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_Users_Roles')
      ALTER TABLE [dbo].[TT_Users] DROP CONSTRAINT [FK_Users_Roles]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_Users Table Updated Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Update dbo.TT_Users Table'
END
GO


BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Updating dbo.TT_CustomerDetails Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO


IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_TT_CustomerDetails_TT_Users')
      ALTER TABLE [dbo].[TT_CustomerDetails] DROP CONSTRAINT [FK_TT_CustomerDetails_TT_Users]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[TT_CustomerDetails]
      ALTER COLUMN [State] [nvarchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NULL
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO


IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_CustomerDetails Table Updated Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Update dbo.TT_CustomerDetails Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Deleting dbo.TT_Priorities Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   DROP TABLE [dbo].[TT_Priorities]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_Priorities Table Deleted Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Delete dbo.TT_Priorities Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Deleting dbo.TT_Statuses Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   DROP TABLE [dbo].[TT_Statuses]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_Statuses Table Deleted Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Delete dbo.TT_Statuses Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Updating dbo.TT_Tasks Table'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO


IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_TT_Tasks_TT_Projects')
      ALTER TABLE [dbo].[TT_Tasks] DROP CONSTRAINT [FK_TT_Tasks_TT_Projects]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   ALTER TABLE [dbo].[TT_Tasks]
      DROP COLUMN [ChangeStatus],
                  [IsOutOfScope]
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

 ALTER TABLE [dbo].[TT_Tasks]
    ADD [CustomData] [ntext] COLLATE Cyrillic_General_CI_AS NULL

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_Tasks Table Updated Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Update dbo.TT_Tasks Table'
END
GO


BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_calcProjectAttachCount Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_calcProjectAttachCount
	(
		@ProjectID int
	)
RETURNS int
AS 
	BEGIN
		DECLARE 
			@taskACount int,
			@projACount int
		
		SET @taskACount = 
		(
			SELECT COUNT(AttachID) 
			FROM TT_Attach att
			INNER JOIN TT_Tasks ts ON ts.TaskID = att.ParentID
			WHERE att.ParentType = 1 AND ts.ProjectID = @ProjectID
		)
		
		SET @projACount = 
		(
			SELECT COUNT(AttachID) 
			FROM TT_Attach att
			INNER JOIN TT_Projects pj ON pj.ProjectID = att.ParentID
			WHERE att.ParentType = 4 AND pj.ProjectID = @ProjectID
		)
		
	RETURN @taskACount + @projACount
	END')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_calcProjectAttachCount Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_calcProjectAttachCount Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_calcProjectMessageCount Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_calcProjectMessageCount
	(
		@ProjectID int
	)
RETURNS int
AS
	BEGIN
		DECLARE @msgCount int
		
		SET @msgCount = 
		(
			SELECT COUNT(MessageID)
			FROM TT_MessageBoard
			WHERE ProjectID = @ProjectID
		)
		
	RETURN @msgCount
	END')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_calcProjectMessageCount Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_calcProjectMessageCount Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_calcProjectTaskCount Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_calcProjectTaskCount
	(
		@ProjectID int,
		@TaskType int
	)
RETURNS int
AS
	BEGIN
		DECLARE @taskCount int
		
		SET @taskCount =
		(
			SELECT 
				COUNT(TaskID)
			FROM TT_Tasks
			WHERE TaskType = @TaskType AND ProjectID = @ProjectID
		)
		
		RETURN @taskCount
	END')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_calcProjectTaskCount Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_calcProjectTaskCount Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_getTaskAttachCount Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_getTaskAttachCount
	(
		@TaskID int
	/*
	@parameter1 datatype = default value,
	@parameter2 datatype
	*/
	)
RETURNS int
AS
	BEGIN
		DECLARE 
			@taskACount int
		
		SET @taskACount = 
		(
			SELECT COUNT(AttachID) 
			FROM TT_Attach att
			INNER JOIN TT_Tasks ts ON ts.TaskID = att.ParentID
			WHERE att.ParentType = 1 AND ts.TaskID = @TaskID
		)
	RETURN @taskACount
	END')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_getTaskAttachCount Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_getTaskAttachCount Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_getTaskMessageCount Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_getTaskMessageCount
	(
		@TaskID int
	)
RETURNS int
AS
	BEGIN
		DECLARE @msgCount int
		
		SET @msgCount = 
		(
			SELECT COUNT(MessageID)
			FROM TT_MessageBoard
			WHERE ParentID = @TaskID AND ParentType = 1
		)
	RETURN @msgCount
	END')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_getTaskMessageCount Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_getTaskMessageCount Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_wflGetLastRecordedDateForTask Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_wflGetLastRecordedDateForTask
	(
		@TaskID int	
	)
RETURNS datetime
AS
	BEGIN
		DECLARE @ld datetime
		
		SET @ld = 
		(
			SELECT TOP 1 EntryDate 
			FROM TT_Workflow 
			WHERE ObjectType=3 AND ObjectID=@TaskID 
			ORDER BY EntryDate DESC
		)
		
	RETURN @ld
	END')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_wflGetLastRecordedDateForTask Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_wflGetLastRecordedDateForTask Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.vwProjects View'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE VIEW dbo.vwProjects
AS
SELECT     pj.ProjectID, pj.Name, pj.Description, pj.EstCompletionDate, pj.EstDuration, pj.CreationDate, pj.Status, pj.BillingType,
	     pj.Amount, pj.IncludeTax, pj.TaxAmount, pj.AutoAssignConfig, pj.Priority, pj.PercentCompleted, pj.BugCategoryID, pj.ReqCategoryID, pj.CustomerID,
pri.Name AS PriorityName, st.StatusName AS StatusName, us.DisplayName AS ManagerDisplayName, cust.CompanyName, 
                      dbo.ep_calcProjectAttachCount(pj.ProjectID) AS AttachCount, dbo.ep_calcProjectTaskCount(pj.ProjectID, 1) AS TaskCount, 
                      dbo.ep_calcProjectTaskCount(pj.ProjectID, 2) AS BugCount, dbo.ep_calcProjectTaskCount(pj.ProjectID, 3) AS RequestCount, 
                      dbo.ep_calcProjectMessageCount(pj.ProjectID) AS MessageCount
FROM         dbo.TT_Projects pj INNER JOIN
                      dbo.DD_Priorities pri ON pri.Number = pj.Priority INNER JOIN
                      dbo.DD_Statuses st ON st.StatusType = ''Project'' AND st.Number = pj.Status INNER JOIN
                      dbo.TT_ProjectMembers mb ON mb.ProjectID = pj.ProjectID AND mb.RoleID = 2 INNER JOIN
                      dbo.TT_Users us ON us.UserID = mb.UserID LEFT OUTER JOIN
                      dbo.TT_CustomerDetails cust ON cust.CustomerID = pj.CustomerID')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.vwProjects View Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.vwProjects View'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.vwTaskWorkflow View'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE VIEW dbo.vwTaskWorkflow
AS
SELECT     ts.TaskID, us.DisplayName, wfl.UserID, wfl.EntryDate, wfl.Action, wfl.CustomAction
FROM         dbo.TT_Workflow wfl INNER JOIN
                      dbo.TT_Tasks ts ON ts.TaskID = wfl.ObjectID INNER JOIN
                      dbo.TT_Users us ON us.UserID = wfl.UserID
WHERE     (wfl.ObjectType = 3)')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.vwTaskWorkflow View Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.vwTaskWorkflow View'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.vwTasks View'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE VIEW dbo.vwTasks
AS 
SELECT     ts.TaskID, ts.Description, ts.ParentID, ts.ProjectID, ts.Status, ts.EstCompletionDate, ts.EstDuration, ts.CategoryID, ts.Title, ts.Billable, 
	     ts.BillType, ts.Amount, ts.StartDate, ts.PercentCompleted, ts.Priority, ts.TaskType, ts.BugType, ts.CustomData,
	         pj.Name AS ProjectTitle, st.StatusName AS StatusName, wfl1.DisplayName AS CreatedByName, wfl1.UserID AS CreatedBy, 
                      wfl2.DisplayName AS CompletedByName, wfl2.UserID AS CompletedBy, dbo.ep_getTaskAttachCount(ts.TaskID) AS AttachCount, 
                      dbo.ep_getTaskMessageCount(ts.TaskID) AS MessageCount, wfl1.EntryDate AS CreationDate, bgt.Name AS BugTypeString, pri.Name AS PriorityName, 
                      dbo.ep_wflGetLastRecordedDateForTask(ts.TaskID) AS LastStatusChangeDate, csd.CustomerID AS CustomerID, csd.CompanyName, 
                      st.TreatAsCompleted, cat.Name AS CategoryName, ddt.TypeName AS TaskTypeName
FROM         dbo.TT_Tasks ts INNER JOIN
                      dbo.TT_Projects pj ON pj.ProjectID = ts.ProjectID LEFT OUTER JOIN
                      dbo.vwTaskWorkflow wfl1 ON wfl1.TaskID = ts.TaskID AND wfl1.Action = 1 LEFT OUTER JOIN
                      dbo.vwTaskWorkflow wfl2 ON wfl2.TaskID = ts.TaskID AND wfl2.Action = 2 AND wfl2.CustomAction = 4 LEFT OUTER JOIN
                      dbo.DD_BugTypes bgt ON bgt.Number = ts.BugType INNER JOIN
                      dbo.DD_Priorities pri ON pri.Number = ts.Priority LEFT OUTER JOIN
                      dbo.TT_CustomerDetails csd ON csd.CustomerID = pj.CustomerID INNER JOIN
                      dbo.TT_Categories cat ON cat.CategoryID = ts.CategoryID INNER JOIN
                      dbo.DD_TaskTypes ddt ON ddt.ID = ts.TaskType INNER JOIN
                      dbo.DD_Statuses st ON st.StatusType = ddt.TypeName AND st.Number = ts.Status')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.vwTasks View Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.vwTasks View'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.vwTaskAssignee View'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE VIEW dbo.vwTaskAssignee
AS 
SELECT     ts.TaskID, ts.Description, ts.ParentID, ts.ProjectID, ts.Status, ts.EstCompletionDate, ts.EstDuration, ts.CategoryID, ts.Title, ts.Billable, 
	     ts.BillType, ts.Amount, ts.StartDate, ts.PercentCompleted, ts.Priority, ts.TaskType, ts.BugType, ts.CustomData,
	     ts.ProjectTitle, ts.StatusName, ts.CreatedByName, ts.CreatedBy, 
                  ts.CompletedByName, ts.CompletedBy, ts.AttachCount, 
                  ts.MessageCount, ts.CreationDate, ts.BugTypeString, ts.PriorityName, 
                  ts.LastStatusChangeDate, ts.CustomerID, ts.CompanyName, 
                  ts.TreatAsCompleted, ts.CategoryName, ts.TaskTypeName,
	     tsa.UserID AS AssignedToId, us.DisplayName AS AssignedToName
FROM         dbo.vwTasks ts LEFT OUTER JOIN
                      dbo.TT_TaskAssignee tsa ON tsa.TaskID = ts.TaskID LEFT OUTER JOIN
                      dbo.TT_Users us ON us.UserID = tsa.UserID')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.vwTaskAssignee View Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.vwTaskAssignee View'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.ep_requestAssignedTasks Function'
GO

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL ON
GO

SET NUMERIC_ROUNDABORT OFF
GO

exec('CREATE FUNCTION dbo.ep_requestAssignedTasks
	(
		@assignedToId int,
		@TaskType int
	)
RETURNS TABLE
AS
	RETURN 
	SELECT * FROM vwTaskAssignee WHERE AssignedToId = @assignedToId AND TaskType=@TaskType')
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.ep_requestAssignedTasks Function Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.ep_requestAssignedTasks Function'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_Notifications Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_NF_Notifications_TT_Projects')
      ALTER TABLE [dbo].[NF_Notifications] WITH NOCHECK ADD CONSTRAINT [FK_NF_Notifications_TT_Projects] FOREIGN KEY ([ProjectID]) REFERENCES [dbo].[TT_Projects] ([ProjectID]) ON DELETE CASCADE
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_Notifications Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_Notifications Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_NotificationDetails Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_NF_NotificationDetails_NF_Actions')
      ALTER TABLE [dbo].[NF_NotificationDetails] ADD CONSTRAINT [FK_NF_NotificationDetails_NF_Actions] FOREIGN KEY ([ActionID]) REFERENCES [dbo].[NF_Actions] ([ActionID]) ON DELETE CASCADE
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_NF_NotificationDetails_NF_Notifications')
      ALTER TABLE [dbo].[NF_NotificationDetails] WITH NOCHECK ADD CONSTRAINT [FK_NF_NotificationDetails_NF_Notifications] FOREIGN KEY ([NotificationID]) REFERENCES [dbo].[NF_Notifications] ([NotificationID]) ON DELETE CASCADE
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_NotificationDetails Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_NotificationDetails Table'
END
GO

BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Creating dbo.NF_NotificationUsers Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_NF_NotificationUsers_NF_Notifications')
      ALTER TABLE [dbo].[NF_NotificationUsers] WITH NOCHECK ADD CONSTRAINT [FK_NF_NotificationUsers_NF_Notifications] FOREIGN KEY ([NotificationID]) REFERENCES [dbo].[NF_Notifications] ([NotificationID]) ON DELETE CASCADE
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_NF_NotificationUsers_TT_Users')
      ALTER TABLE [dbo].[NF_NotificationUsers] ADD CONSTRAINT [FK_NF_NotificationUsers_TT_Users] FOREIGN KEY ([UserID]) REFERENCES [dbo].[TT_Users] ([UserID]) ON DELETE CASCADE
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.NF_NotificationUsers Table Added Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Add dbo.NF_NotificationUsers Table'
END
GO


BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Updating dbo.TT_Users Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_Users_Roles')
      ALTER TABLE [dbo].[TT_Users] ADD CONSTRAINT [FK_Users_Roles] FOREIGN KEY ([RoleID]) REFERENCES [dbo].[TT_Roles] ([RoleID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_Users Table Updated Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Update dbo.TT_Users Table'
END
GO


BEGIN TRANSAction
GO
SET TRANSAction ISOLATION LEVEL SERIALIZABLE
GO

PRINT 'Updating dbo.TT_Tasks Table'
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_TT_Tasks_TT_Categories')
      ALTER TABLE [dbo].[TT_Tasks] ADD CONSTRAINT [FK_TT_Tasks_TT_Categories] FOREIGN KEY ([CategoryID]) REFERENCES [dbo].[TT_Categories] ([CategoryID])
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
   IF NOT EXISTS (SELECT name FROM sysobjects WHERE name = N'FK_TT_Tasks_TT_Projects')
      ALTER TABLE [dbo].[TT_Tasks] ADD CONSTRAINT [FK_TT_Tasks_TT_Projects] FOREIGN KEY ([ProjectID]) REFERENCES [dbo].[TT_Projects] ([ProjectID]) ON DELETE CASCADE
GO

IF @@ERROR <> 0
   IF @@TRANCOUNT = 1 ROLLBACK TRANSAction
GO

IF @@TRANCOUNT = 1
BEGIN
   PRINT 'dbo.TT_Tasks Table Updated Successfully'
   COMMIT TRANSAction
END ELSE
BEGIN
   PRINT 'Failed To Update dbo.TT_Tasks Table'
END
GO


insert into DD_TaskTypes(TypeName, ID) values('Task', 1)
insert into DD_TaskTypes(TypeName, ID) values('Bug', 2)
insert into DD_TaskTypes(TypeName, ID) values('Request', 3)
GO

-- update ReportGenerator datasource data

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, QUOTED_IDENTIFIER, CONCAT_NULL_YIELDS_NULL, XACT_ABORT ON
GO
SET NUMERIC_ROUNDABORT OFF
GO

BEGIN TRANSAction

ALTER TABLE [dbo].[RG_DatasourceField] NOCHECK CONSTRAINT [FK_RG_DataSourceField_RG_DataSource]

DELETE FROM [dbo].[RG_DatasourceField] WHERE [DatasourceFieldID] = 181
DELETE FROM [dbo].[RG_DatasourceField] WHERE [DatasourceFieldID] = 183
DELETE FROM [dbo].[RG_DatasourceField] WHERE [DatasourceFieldID] = 193
UPDATE [dbo].[RG_Datasource] SET [QueryText] = 'SELECT pj.*, stat.StatusName, prio.Name as PriorityName
FROM TT_Projects pj
INNER JOIN DD_Statuses as stat
ON stat.Number = pj.Status 
INNER JOIN DD_Priorities prio
ON prio.Number = pj.Priority
WHERE stat.StatusType=''Project'' AND pj.ProjectID IN ?' WHERE [DatasourceID] = 1
UPDATE [dbo].[RG_Datasource] SET [QueryText] = 'SELECT ts.*, proj.Name as ProjectName, stat.StatusName, cat.Name as CategoryName, prior.Name as PriorityName,
CASE WHEN ts.TaskType = 1 THEN ''{task}'' WHEN ts.TaskType = 2 THEN ''{bug}'' ELSE ''{request}'' END as TaskTypeName
								FROM TT_Tasks ts
								INNER JOIN TT_Projects proj
								ON proj.ProjectID = ts.ProjectID
								INNER JOIN DD_Statuses stat
								ON stat.Number = ts.Status
								INNER JOIN TT_Categories cat
								ON cat.CategoryID = ts.CategoryID
								INNER JOIN DD_Priorities prior
								ON prior.Number = ts.Priority 
								WHERE stat.StatusType = CASE WHEN ts.TaskType = 1 THEN ''Task'' WHEN ts.TaskType = 2 THEN ''Bug'' ELSE ''Request'' END
								AND proj.ProjectID IN ?
								UNION ALL

SELECT ts.*, proj.Name as ProjectName, stat.StatusName, cat.Name as CategoryName, prior.Name as PriorityName,
CASE WHEN ts.TaskType = 1 THEN ''{task}'' WHEN ts.TaskType = 2 THEN ''{bug}'' ELSE ''{request}'' END as TaskType
								FROM TT_Tasks ts
								INNER JOIN TT_Projects proj
								ON proj.ProjectID = ts.ProjectID
								INNER JOIN DD_Statuses stat
								ON stat.Number = ts.Status
								INNER JOIN TT_Categories cat
								ON cat.CategoryID = ts.CategoryID
								INNER JOIN DD_Priorities prior
								ON prior.Number = ts.Priority 
								WHERE stat.StatusType = CASE WHEN ts.TaskType = 1 THEN ''Task'' WHEN ts.TaskType = 2 THEN ''Bug'' ELSE ''Request'' END
								AND proj.ProjectID IN ?
								AND EXISTS (SELECT * FROM TT_TaskAssignee ta WHERE 
								ta.TaskID = ts.TaskID
								AND ta.UserID = ?
								)' WHERE [DatasourceID] = 2
UPDATE [dbo].[RG_Datasource] SET [QueryText] = 'SELECT ts.*, proj.Name as ProjectName, bgtp.Name as BugTypeName, stat.StatusName, prior.Name as PriorityName
								FROM TT_Tasks ts
								INNER JOIN TT_Projects proj
								ON proj.ProjectID = ts.ProjectID
								INNER JOIN DD_Statuses stat
								ON stat.Number = ts.Status AND stat.StatusType = ''Bug''
								LEFT OUTER JOIN DD_BugTypes bgtp
								ON bgtp.Number = ts.BugType
								INNER JOIN DD_Priorities prior
								ON prior.Number = ts.Priority 
								WHERE ts.TaskType = 2 
								AND proj.ProjectID IN ?
								UNION ALL

SELECT ts.*, proj.Name as ProjectName, bgtp.Name as BugTypeName, stat.StatusName, prior.Name as PriorityName
								FROM TT_Tasks ts
								INNER JOIN TT_Projects proj
								ON proj.ProjectID = ts.ProjectID
								INNER JOIN DD_Statuses stat
								ON stat.Number = ts.Status AND stat.StatusType = ''Bug''
								LEFT OUTER JOIN DD_BugTypes bgtp
								ON bgtp.Number = ts.BugType
								INNER JOIN DD_Priorities prior
								ON prior.Number = ts.Priority 
								WHERE ts.TaskType = 2
								AND proj.ProjectID IN ?
								AND EXISTS (SELECT * FROM TT_TaskAssignee ta WHERE 
								ta.TaskID = ts.TaskID
								AND ta.UserID = ?
								)' WHERE [DatasourceID] = 4
UPDATE [dbo].[RG_Datasource] SET [QueryText] = 'SELECT ts.*, proj.Name as ProjectName, stat.StatusName, prior.Name as PriorityName
								FROM TT_Tasks ts
								INNER JOIN TT_Projects proj
								ON proj.ProjectID = ts.ProjectID
								INNER JOIN DD_Statuses stat
								ON stat.Number = ts.Status AND stat.StatusType = ''Request''
								INNER JOIN DD_Priorities prior
								ON prior.Number = ts.Priority 
								WHERE ts.TaskType = 3 
								AND proj.ProjectID IN ?
								UNION ALL

SELECT ts.*, proj.Name as ProjectName, stat.StatusName, prior.Name as PriorityName
								FROM TT_Tasks ts
								INNER JOIN TT_Projects proj
								ON proj.ProjectID = ts.ProjectID
								INNER JOIN DD_Statuses stat
								ON stat.Number = ts.Status AND stat.StatusType = ''Request''
								INNER JOIN DD_Priorities prior
								ON prior.Number = ts.Priority 
								WHERE ts.TaskType = 3
								AND proj.ProjectID IN ?
								AND EXISTS (SELECT * FROM TT_TaskAssignee ta WHERE 
								ta.TaskID = ts.TaskID
								AND ta.UserID = ?
								)' WHERE [DatasourceID] = 5
UPDATE [dbo].[RG_DatasourceField] SET [FieldName] = 'PriorityName' WHERE [DatasourceFieldID] = 11
UPDATE [dbo].[RG_DatasourceField] SET [FieldName] = 'TaskID' WHERE [DatasourceFieldID] = 170
UPDATE [dbo].[RG_DatasourceField] SET [FieldName] = 'StartDate' WHERE [DatasourceFieldID] = 176
UPDATE [dbo].[RG_DatasourceField] SET [FieldName] = 'TaskID' WHERE [DatasourceFieldID] = 186
UPDATE [dbo].[RG_DatasourceField] SET [FieldName] = 'StartDate' WHERE [DatasourceFieldID] = 191
SET IDENTITY_INSERT [dbo].[RG_DatasourceField] ON
INSERT INTO [dbo].[RG_DatasourceField]([DatasourceFieldID], [DatasourceID], [FieldName], [DisplayName], [Datatype], [DataFormat], [Width], [Aggregation]) VALUES (211, 2, 'TaskTypeName', 'Task Type', 2, NULL, 10, 0)
SET IDENTITY_INSERT [dbo].[RG_DatasourceField] OFF
ALTER TABLE [dbo].[RG_DatasourceField] CHECK CONSTRAINT [FK_RG_DataSourceField_RG_DataSource]

COMMIT TRANSAction

-- end of update RG data

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[TT_Version]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
drop table [dbo].[TT_Version]
GO

CREATE TABLE [dbo].[TT_Version] (
	[Major] [int] NOT NULL ,
	[Minor] [int] NOT NULL ,
	[Build] [int] NOT NULL ,
	[MinorBuild] [int] NOT NULL 
) ON [PRIMARY]
GO

INSERT INTO NF_Actions (Operation, Target) VALUES ('Add', 'Task')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Edit', 'Task')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Complete', 'Task')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Delete', 'Task')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Add', 'Bug')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Edit', 'Bug')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Complete', 'Bug')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Delete', 'Bug')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Add', 'Request')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Edit', 'Request')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Complete', 'Request')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Delete', 'Request')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Add', 'Attachment')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Delete', 'Attachment')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Add', 'Message')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Edit', 'Message')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Delete', 'Message')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Assign', 'Task')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Assign', 'Bug')
INSERT INTO NF_Actions (Operation, Target) VALUES ('Assign', 'Request')
GO

INSERT INTO TT_Version(Major,Minor,Build,MinorBuild) VALUES (4,0,1,1)
